/*******************************************************************************
 output_bucket.h - A library for writing memory / file buckets.

 Copyright (C) 2007-2017 CodeShop B.V.
 http://www.code-shop.com

 For licensing see the LICENSE file
******************************************************************************/ 

#ifndef OUTPUT_BUCKET_H_AKW
#define OUTPUT_BUCKET_H_AKW

#include "mod_streaming_export.h"
#include "mp4_error.h"
#include <inttypes.h>
#include <stddef.h>

#ifdef __cplusplus
extern "C" {
#endif

struct MP4_DLL_EXPORT bucket_t
{
  struct bucket_t* prev_;
  struct bucket_t* next_;

  uint64_t offset_;

  // the size of the data. If it's unknown the value is UINT64_MAX.
  uint64_t size_;

  // bucket_memory_t*, bucket_file_t*, bucket_xfrm_t*, etc...
  struct bucket_base_t* data_;
};
typedef struct bucket_t bucket_t;

MP4_DLL_EXPORT uint32_t get_bucket_type(bucket_t const* bucket);
MP4_DLL_EXPORT int is_bucket_type_heap(bucket_t const* bucket);
MP4_DLL_EXPORT int is_bucket_type_file(bucket_t const* bucket);
MP4_DLL_EXPORT int is_bucket_type_http(bucket_t const* bucket);
MP4_DLL_EXPORT int is_bucket_type_xfrm(bucket_t const* bucket);
MP4_DLL_EXPORT int is_bucket_type_iobf(bucket_t const* bucket);
MP4_DLL_EXPORT int is_bucket_type_pipe(bucket_t const* bucket);

MP4_DLL_EXPORT void bucket_destroy(bucket_t* bucket);

MP4_DLL_EXPORT fmp4_result bucket_read(bucket_t* bucket,
                                       uint8_t const** buf, size_t* size);

MP4_DLL_EXPORT fmp4_result bucket_write(bucket_t* bucket,
                                        uint8_t** buf, size_t* size);

MP4_DLL_EXPORT fmp4_result bucket_split(bucket_t* bucket, uint64_t size);

MP4_DLL_EXPORT fmp4_result bucket_copy(bucket_t* src, bucket_t** dst);

MP4_DLL_EXPORT void bucket_clone(bucket_t* src, bucket_t** dst);

MP4_DLL_EXPORT extern char const* cache_control_no_cache;

struct buckets_t
{
  char const* content_type_;

  // when "cache_control" is set, it is copied to the "Cache-Control" header
  // field. Otherwise the "Cache-Control" may be based on the "expires_at".
  char const* cache_control_;

  // When the "cache_control" is not set to "no-cache", then an "ETag" header
  // field is written.
  // If no "revision_id" is given (0), then the "ETag" is based on the last
  // modified time, otherwise the "revision_id" is used as "ETag".
  uint64_t revision_id_;

  // when set, the "updated_at" is copied to the "Last-Modified" header field.
  uint64_t updated_at_;

  // Creates a "Cache-Control max-age=%" and "Expires" header field by using
  // the "expires_at".
  uint64_t expires_at_;

  // when "location" is set, it is copied to the "Location" header field and
  // a redirect is issued.
  char* location_;

  // the bucket list that is output
  bucket_t* bucket_;

  //
  char x_usp_header1[256];
  char x_usp_header2[256];
  char link_[512];
};
typedef struct buckets_t buckets_t;

MP4_DLL_EXPORT void bucket_insert_tail(buckets_t* buckets, bucket_t* bucket);

MP4_DLL_EXPORT bucket_t* bucket_heap_create(uint8_t const* buf, uint64_t size);

// utility functions

MP4_DLL_EXPORT extern void bucket_exit(bucket_t* bucket);
MP4_DLL_EXPORT extern void bucket_remove(bucket_t* bucket);
MP4_DLL_EXPORT extern void bucket_delete(bucket_t* bucket);

// returns the size of the buckets by accumulating the size of all the buckets.
// If a bucket with unknown length (UINT64_MAX) is encountered, then UINT64_MAX
// is returned.

MP4_DLL_EXPORT extern
uint64_t buckets_size(buckets_t const* buckets, bucket_t const* end_bucket);

MP4_DLL_EXPORT extern void buckets_clear(buckets_t* buckets);

MP4_DLL_EXPORT
void bucket_file_read(bucket_t* bucket, char const** filename,
                      uint64_t* offset, uint64_t* size, uint32_t max_size);

MP4_DLL_EXPORT char* file_url_to_path(char* filename);

#ifdef __cplusplus
} /* extern C definitions */
#endif

#endif // OUTPUT_BUCKET_H_AKW

// End Of File

